package lava.security;



/******************************************************************************
Input/output interface for abstract text transforms.

<p><b>Details:</b>  An <code>ITextTransform</code> is a filter with the following basic properties:</p>

<ul>
	<li>It consumes, as input, a stream of symbols.</li>
	<li>It produces, as output, a stream of symbols, based on the input and transformation model.</li>
	<li>Output symbols will not necessarily be produced at the same rate as input symbols are consumed.</li>
</ul>

<p>Symbols are basically integers, or bit vectors.  The meaning that an <code>ITextTransform</code> assigns to them is not specified by this interface and is implementation-<wbr>specific.  It is common to refer to a stream of symbols as "text"; please don't confuse this usage with the more generic meaning of the same word.</p>

<p>Some common examples of text transforms include:</p>

<ul>
	<li>An encryption algorithm that accepts a stream of 8-bit bytes and produces an equally long stream of 8-bit bytes (such as a "one-time pad" encryption algorithm).</li>
	<li>An encyrption algorithm that accepts a stream of 64-bit blocks and produces an equally long stream of 64-bit blocks (such as DES in ECB mode).</li>
	<li>A digest algorithm that accepts an arbitrarily long sequence of bytes to produce a short, fixed-length sequence of bytes representing the input's "signature."</li>
</ul>

<p>Most text transforms consume bytes and produce bytes.  However, there are a few transforms, such as DES in OFB or CFB mode, which can actually be configured to process symbols streams whose widths vary from 1 to 64 bits.  Hence, this interface allows input and output symbols to be represented with <code>byte</code>s, <code>short</code>s, <code>int</code>s, or <code>long</code>s.</p>

<p>All input and output text is communicated by means of arrays of these basic integer types (i.e., <code>byte[]</code>, <code>short[]</code>, <code>int[]</code>, or <code>long[]</code>).  In order to support efficient buffering, the client must supply both the source/input and destination/output arrays when calling the methods of this interface.</p>

<p>A valid <code>ITextTransform</code> implementation must accept, as input, any array whose base type is wide enough to hold a whole input symbol.  Similarly, a valid <code>ITextTransform</code> must also be willing to accept, as an output destination, any array whose base type is wide enough to store a complete output symbol.  Neither input symbols nor output symbols are ever split among multiple elements or packed into single elements.  Hence, if the input or output text is comprised of 8-bit symbols, it is wasteful to use <code>int[]</code> arrays as buffers, but it nevertheless allowed.  Similarly, if a transformation's output text is comprised of 64-bit symbols, a valid implementation may refuse to store the symbols in <code>byte[]</code> buffers.</p>

<p>If the input array's base type is wider than the input symbol width, a valid implementation may assume that the unused MSBs are 0.  Similarly, if the output array's base type is wider than the output symbol width, a valid implementation is required to place 0s in the unused MSBs when storing output in the provided destination array.</p>

<p>For any given text transform, the symbol widths of the input and output text can be discovered by calling, respectively, <code>getTransformInputWidth</code> and <code>getTransformOutputWidth</code>.</p>

<p>Because the state of an <code>ITextTransform</code> instance may depend on previous input history, in general is not advisable to transform multiple streams in parallel through the same instance.  However, because the <code>beginTransform</code> method makes it possible to reset a given <code>ITextTransform</code> instance for a new data stream, processing multiple streams in serial with the same instance is acceptable.  Implementations are not required to be thread-<wbr>safe.</p>

<p>The primary methods for performing text transformations are:</p>

<ol>
	<li><code>beginTransform</code> -- reset transform (call before processing first symbol)
	<li><code>continueTransform</code> -- consume and produce symbols (call repeatedly)
	<li><code>completeTransform</code> -- finish delayed symbol processing (call after all symbols have been processed)
</ol>

<p>To perform an abstract text transformation correctly, these three methods should be called in the order listed.</p>

<form>
<p>Clip-source:<br>
<textarea rows=5 cols=80 onChange="this.form.reset();">
No clip-source available.
</textarea></form>

@since 1999.08.08
@version 1999.08.18
******************************************************************************/

public interface ITextTransform
{



	/**********************************************************************
	Resets transform.

	<p><b>Details:</b>  <code>beginTransform</code> initializes this transform in preparation for processing a new input text stream.</p>
	**********************************************************************/

	void beginTransform ();



	/**********************************************************************
	Processes text.

	<p><b>Details:</b> <code>continueTransform</code> consumes input text and produces output text.  Input is retrieved from the provided source array (<var>src</var>), while output is stored in the provided destination array (<var>dest</var>).  The return value normally indicates the number of symbols written into the destination array (see note below).</p>

	<p>Depending on the particular transformation represented by this implementation, the number of output symbols produced per call may always be equal to the number of input symbols consumed (for example, if the transformation performs text compression).  Similarly, a call to this method may cause the production of more elements than the supplied destination array can store.  It is the caller's responsibility to make sure the supplied destination array is large enough to receive the symbols produced.</p>

	<p>If the destination array is too short, the implementation may, at its option, discard the overflow.  However, an implementation may also choose to retain the overflow portion and make it available in a subsequent call to this method.  (Check the implementation documentation to see which approach is taken.)  If the overflow is discarded, the return value must indicate the number of symbols written into the array plus the number of overflow symbols discarded.</p>

	<p>It is legal to call this method with <var>src</var> set to <code>null</code> so long as length is also set to 0.  This might be useful in situations where a transform is expected to produce output symbols even when the caller has no input symbols to supply (i.e., to recover buffered overflow from a previous call).</p>

	<p><code>continueTransform</code> always writes as much data into <var>dest</var> as can be computed from the current input history.  However, because some data may not be computable until the transformation sees the last symbol (such as in checksum algorithms), it may be necessary to obtain the rest of the output data by calling <code>completeTransform</code>.</p>

	<p>The results of <code>continueTransform</code> are undefined if it is called after <code>completeTransform</code> but before <code>beginTransform</code>.</p>

	@param src the source array
	@param dest the destination array
	@param length number of symbols to process
	@return number of elements written into dest or discarded
	**********************************************************************/

	int continueTransform (Object src, Object dest, int length);



	/**********************************************************************
	Completes transform.

	<p><b>Details:</b>  <code>completeTransform</code> signals to the implementation that there are no more input symbols to consume.  This method completes the transform and gives the implementation an opportunity to perform final processing that may produce output symnbols.  If output symbols are generated, they are stored in the supplied destination array (<var>dest</var>).  The return value indicates the number output symbols stored.</p>

	<p>Buffer overflow similar to what was described in <code>continueTransform</code> may also occur with <code>completeTransform</code>.  Again, it is up to the caller to make sure that <code>dest</code> is large enough to hold the result, and the implementation may discard the remainder.  If overflow occurs and symbols are discarded, the return value is the total number of stored and discarded symbols.  Optionally, an implementation may also capture overflow and make it available to the caller through repeated calls to this method.  No more symbols are available when <code>completeTransform</code> returns 0.</p>

	@param dest the destination array
	@return number of elements written into dest or discarded
	**********************************************************************/

	int completeTransform (Object dest);



	/**********************************************************************
	Returns input symbol width.

	<p><b>Details:</b>  This method returns the width, in bits, of the transform's input symbol.  Use this method to determine what array types can be used for storing input.</p>

	@return input symbol width
	@since 1999.08.18
	**********************************************************************/

	int getTransformInputWidth ();



	/**********************************************************************
	Returns output symbol width.

	<p><b>Details:</b>  This method returns the width, in bits, of the transform's output symbol.  Use this method to determine what array types can be used for storing output.</p>

	@return output symbol width
	@since 1999.08.18
	**********************************************************************/

	int getTransformOutputWidth ();



}



