///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security;



import java.lang.reflect.Array;
import lava.konst.LONG;
import lava.array.ArrayCaster;



/******************************************************************************
Identity transform.

<p><b>Details:</b>  This class is the identity text transform, or a text transform which produces an output symbol sequence equivalent to the input symbol sequence it consumes.  As with any implementation of <code>ITextTransform</code>, the input and output array types may be different.  Regardless of the array types, <code>IdentityTextTransform</code> converts the input symbols into symbols that can be stored in the destination array, provided that the destination array's type is wide enough for the symbol size.</p>

<p>This class implements the <code>ITextTransform</code> API.  See the {@link ITextTransform ITextTransform documentation} for more information.</p>

<p>An additional feature of this transform is that if any of the padding bits in the input block are not 0, they will be zeroed in the output.  (Note that this behavior is provided as a convenience and is <em>not</em> required by the <code>ITextTransform</code> interface.  In many ways, this "extra feature" makes this implementation similar to the <code>ArrayCaster</code> classes in package <code>lava.array</code>.)</p>

@author Sharky
@since 1999.08.16
******************************************************************************/

public final class IdentityTextTransform
	implements ITextTransform
{



	private final int width;



	/**********************************************************************
	Sets symbol width.

	<p><b>Details:</b>  This constructor initializes an identity transform for symbols of the given width (<var>width</var>).</p>

	@param width the symbol width
	**********************************************************************/

	public IdentityTextTransform (int width)
	{
		if (width < 1 || width > 64)
			throw new IllegalArgumentException ("width=" + width);
		this . width = width;
	}



	/**********************************************************************
	No operation.

	<p><b>Details:</b>  <code>beginTransform</code> does absolutely nothing in this implementation.</p>
	**********************************************************************/

	public void beginTransform () {}



	/**********************************************************************
	Converts symbols.

	<p><b>Details:</b>  <code>continueTransform</code> performs the identity transform described in the introduction to this class.</p>

	<p>Overflow symbols are discarded.  (See overflow discussion in <code>ITextTransform</code>.)</p>

	@param input the input text
	@param output the output text
	@return number of symbols generated

	@version 2001.03.06
	**********************************************************************/

	public int continueTransform (Object input, Object output, int len)
	{
		final int inwidth = ArrayCaster.getPrimitiveElementType (input) * 8;
		int outlen;
		switch (inwidth)
		{
		case 8:
			outlen = ((byte[]) output) . length;
			break;
		case 16:
			outlen = ((short[]) output) . length;
			break;
		case 32:
			outlen = ((int[]) output) . length;
			break;
		case 64:
			outlen = ((long[]) output) . length;
			break;
		default:
			throw new IllegalArgumentException ("output");
		}
		if (inwidth < width)
			throw new IllegalArgumentException ("input");
		final int outwidth = ArrayCaster.getPrimitiveElementType (output) * 8;
		if (outwidth < width)
			throw new IllegalArgumentException ("output");
		final long mask = -1L >>> (64 - width);
		long symbol;
		for (int i = 0; i < len; ++ i)
		{
			switch (inwidth)
			{
			case 8:
				symbol = ((byte[]) input) [i];
				break;
			case 16:
				symbol = ((short[]) input) [i];
				break;
			case 32:
				symbol = ((int[]) input) [i];
				break;
			case 64:
				symbol = ((long[]) input) [i];
				break;
			default:
				throw new IllegalArgumentException ("input");
			}
			if (i >= outlen)
				continue;
			symbol &= mask;
			switch (outwidth)
			{
			case 8:
				((byte[]) output) [i] = (byte) symbol;
				break;
			case 16:
				((short[]) output) [i] = (short) symbol;
				break;
			case 32:
				((int[]) output) [i] = (int) symbol;
				break;
			case 64:
				((long[]) output) [i] = symbol;
				break;
			default:
				throw new lava.UnreachableCodeException ();
			}
		}
		return len;
	}



	/**********************************************************************
	No operation.

	<p><b>Details:</b> <code>completeTransform</code> does absolutely nothing in this implementation.</p>

	@param ignored ignored
	@return 0
	**********************************************************************/

	public final int completeTransform (Object ignored)
	{
		return 0;
	}



	/**********************************************************************
	Returns symbol width.

	@return width parameter from constructor
	**********************************************************************/

	public final int getTransformInputWidth ()
	{
		return width;
	}



	/**********************************************************************
	Returns symbol width.

	@return width parameter from constructor
	**********************************************************************/

	public final int getTransformOutputWidth ()
	{
		return width;
	}



}



