package lava.security;



import java.io.InputStream;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import lava.array.ArrayFiller;
import lava.io.IoCloser;



/******************************************************************************
Key generation toolbox.

<p><b>Details:</b> <code>KeyGenerator</code> contains a variety of functions for generating keys.</p>

@since 2000.03.08
@author Sharky
******************************************************************************/

public class KeyGenerator
{



	/**********************************************************************
	Generates extended hash.

	<p><b>Details:</b>  getInterleavedHash generates an "extended hash" by computing several hashes at once and concatenating them together.  The data to be hashed is supplied by <var>iopIn</var>, and the number of simultaneous subhashes to compute is given by <var>inMultiplicity</var>.  Each subhash takes its turn drawing bytes, one at a time, from <var>iopIn</var> until it runs out of bytes.  The resulting hashes are then concatenated together.  The hash function is given by <var>isAlgorithm</var>.  Any hash algorithm supported by <code>java.security.MessageDigest</code> is supported by this function.  For most Java installations, this includes "SHA" and "MD5".</p>

	@param iopIn the input data
	@param isAlgorithm the hash function
	@param inMultiplicity number of simultaneous subhashes
	@return concatenated hash
	@exception IOException if an I/O error occurs
	**********************************************************************/

	public static byte[] getInterleavedHash
	(	InputStream iopIn,
		String isAlgorithm,
		int inMultiplicity
	)
		throws IOException
	{
		if (inMultiplicity < 1)
			throw new IllegalArgumentException ("inMultiplicity=" + inMultiplicity);
		MessageDigest[] vpDigests = new MessageDigest [inMultiplicity];
		for (int vnI = 0; vnI < inMultiplicity; ++ vnI)
			try
			{
				vpDigests [vnI] = MessageDigest.getInstance (isAlgorithm);
			}
			catch (NoSuchAlgorithmException e)
			{
				throw new IllegalArgumentException ("isAlgorithm=" + isAlgorithm);
			}
		try
		{
			int vnI = 0;
			while (true)
			{
				int vnByte = iopIn . read ();
				if (vnByte < 0)
					break;
				vpDigests [vnI] . update ((byte) vnByte);
				vnI = (vnI + 1) % inMultiplicity;
			}
		}
		finally
		{
			IoCloser.close (iopIn);
		}
		byte[][] vppbSums = new byte [inMultiplicity] [];
		int vnConcatenatedLength = 0;
		for (int vnI = 0; vnI < inMultiplicity; ++ vnI)
		{
			byte[] vpbSum = vpDigests [vnI] . digest ();
			vnConcatenatedLength += vpbSum . length;
			vppbSums [vnI] = vpbSum;
		}
		byte[] vnConcatenatedSum = new byte [vnConcatenatedLength];
		vnConcatenatedLength = 0;
		for (int vnI = 0; vnI < inMultiplicity; ++ vnI)
		{
			byte[] vpbSum = vppbSums [vnI];
			int vnSumLength = vpbSum . length;
			System.arraycopy
			(	vpbSum,
				0,
				vnConcatenatedSum,
				vnConcatenatedLength,
				vnSumLength
			);
			vnConcatenatedLength += vnSumLength;
		}
		return vnConcatenatedSum;
	}



	/**********************************************************************
	Compresses bit vector.

	<p><b>Details:</b>  compressBits compresses the given bit vector (<var>ipbSrc</var>, <var>inSrcLen</var>) into a bit vector of the requested size (<var>inDesLen</var>).  The result is stored in the supplied destination array (<var>opbDest</var>), if one is given, or in the return value.</p>

	<p>The compression is achieved by linearly mapping each bit in the source vector to the destination vector, by means of the following scaling function:</p>

	<blockquote>
		<var>dest_index</var> = <var>source_index</var> * <var>dest_size</var> / <var>source_size</var><br>
		(using integer arithmatic)
	</blockquote>

	<p>Before the mapping is applied, the destination vector is filled with zeros.  Then, for each bit in the source vector that is set to 1, the mapped bit in the destination vector is toggled.</p>

	<p>If a destination array is not supplied (i.e., <var>obpDest</var> == <code>null</code>), then one is created automatically and returned.</p>

	<p><var>ibpSrc</var> and <var>opbDest</var> are given in <em>bits</em>, not <em>bytes</em>.  Bit zero in <var>ipbSrc</var> is the least significant bit of <var>ipbSrc</var> [<var>inSrcLen</var> - 1].  Similarly, bit zero in <var>opbDest</var>, if supplied, will be the least significant bit of <var>opbDest</var> [<var>inDestLen</var> - 1].  Extra bits in <var>ibpSrc</var> are ignored, and unused bits in <var>opbDest</var> are set to zero.</p>

	@param ipbSrc the source vector
	@param inSrcLen length of source vector
	@param obpDest destination for result
	@param inDestLen length to compress to
	@return the compressed vector
	**********************************************************************/

	public static byte[] compressBits (byte[] ipbSrc, int inSrcLen, byte[] opbDest, int inDestLen)
	{
		if (opbDest == null)
			opbDest = new byte [(inDestLen + 7) / 8];
		else
			ArrayFiller.fill (opbDest, (byte) 0);
		for (int vnI = 0; vnI < inSrcLen; ++ vnI)
			if (getBitBe (ipbSrc, vnI))
				toggleBitBe (opbDest, vnI * inDestLen / inSrcLen);
//System.out.println(StringEncoder.bytesToHexString(opbDest));
		return opbDest;
	}



	private static void setBitBe (byte[] ipbArray, int inPosition, boolean ibValue)
	{
		int vnIndex = ipbArray . length - 1 - inPosition / 8;
		int vnShift = inPosition % 8;
		int vnMask = 1 << vnShift;
		if (ibValue)
			ipbArray [vnIndex] |= vnMask;
		else
			ipbArray [vnIndex] &= ~ vnMask;
	}



	private static boolean getBitBe (byte[] ipbArray, int inPosition)
	{
		int vnIndex = ipbArray . length - 1 - inPosition / 8;
		int vnShift = inPosition % 8;
		int vnMask = 1 << vnShift;
		return (ipbArray [vnIndex] & vnMask) != 0;
	}



	private static void toggleBitBe (byte[] ipbArray, int inPosition)
	{
		int vnIndex = ipbArray . length - 1 - inPosition / 8;
		int vnShift = inPosition % 8;
		int vnMask = 1 << vnShift;
		ipbArray [vnIndex] ^= vnMask;
	}



}



