///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



import lava.konst.INT;
import lava.konst.LONG;
import lava.security.IBlockCipher64;
import lava.security.ITextTransform;



/******************************************************************************
Base class for Cipher Block Chaining (CBC) coders.

<p><b>Details:</b>  Cipher Block Chaining (CBC) is a special operating mode of DES used to enhance the block cipher's security.  CBC is described in FIPS PUB 81.  This class contains definitions common to both the encryptor and the decryptor.</p>

<p>Unlike DES in electronic codebook (ECB) mode, a CBC DES cipher machine has a state that mutates as it processes data.  Therefore, it is inappropriate to encrypt or decrypt multiple streams with the same <code>DesCbcCoder</code> instance.  A single instance should be used for encrypting only or decrypting only, but not both, and should only be used to process one stream.</p>

<p>The chaining mechanism used by CBC can be applied to <em>any</em> 64-bit block cipher, not just DES.  Hence, although this class was designed to be used primarily with <code>DesEngine</code>, it can actually be used with other <code>IBlockCipher64</code> implementations as well.</p>

<p>This class conforms to the <code>ITextTransform</code> API.  However, because the input and output symbol width is 64 bits, only long arrays can be used with the {@link ITextTransform#continueTransform <code>continueTransform</code>} method.</p>

<p>For efficiency reasons, this class and its subclasses are not thread-<wbr>safe.</p>

<p><small>This base class is not usable to the application programmer in any way and is public only to make the JavaDoc comments visible.</small></p>

@author Sharky
@since 1999.08.16
******************************************************************************/

public abstract class DesCbcCoder
	implements ITextTransform
{



	/**********************************************************************
	Encryption engine.

	<p><b>Details:</b>  <var>engine</var> is the block cipher specified at construction time.  This can be a normal DES function, a triple-DES function, or any other type of encryption function that conforms to the <code>IBlockCipher64</code> interface.</p>
	**********************************************************************/

	final IBlockCipher64 engine;



	/**********************************************************************
	Initialization vector.

	<p><b>Details:</b>  This value is specified at construction time.</p>
	**********************************************************************/

	final long iv;



	/**********************************************************************
	Specifies block encryption function and initialization vector.

	<p><b>Details:</b>  This constructor initializes a new instance with the given block cipher and initialization vector.</p>

	@param engine the block cipher
	@param iv the initialization vector
	**********************************************************************/

	DesCbcCoder (IBlockCipher64 engine, long iv)
	{
		this . engine = engine;
		this . iv = iv;
	}



	/**********************************************************************
	ITextTransform interface method.

	<p><b>Details:</b>  See <code>ITextTransform</code> for a description of this interface method's contract.</p>

	<p>This implementation does not produce any output symbols and always returns 0.</p>

	@param output ignored
	@return 0
	**********************************************************************/

	public final synchronized int completeTransform (Object output)
	{
		return 0;
	}



	/**********************************************************************
	ITextTransform interface method.

	<p><b>Details:</b>  See <code>ITextTransform</code> for a description of this interface method's contract.</p>

	<p>This implementation returns 64.</p>

	@return 64
	**********************************************************************/

	public final int getTransformInputWidth ()
	{
		return 64;
	}



	/**********************************************************************
	ITextTransform interface method.

	<p><b>Details:</b>  See <code>ITextTransform</code> for a description of this interface method's contract.</p>

	<p>This implementation returns 64.</p>

	@return 64
	**********************************************************************/

	public final int getTransformOutputWidth ()
	{
		return 64;
	}



}



