///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



import lava.security.IBlockCipher64;
import lava.security.ITextTransform;



/******************************************************************************
Encrypts/decrypts using DES in ECB mode.

<p><b>Details:</b> A <code>DesEcbCoder</code> performs encryption/decryption using DES in ECB (Electronic CodeBook) mode.  ECB mode is the standard 64-bit block encryption/decryption mode defined in FIPS PUB 46.  DES-ECB is a stateless "dictionary" cipher that converts the same input symbol to the same output symbol every time, regardless of input history.  (The dictionary is selected by the key.)  Because the input and output symbols are 64 bit blocks, only <code>long</code> arrays are supported in this class' <code>ITextTransform</code> interface methods.</p>

<p>In order to encrypt/decrypt <code>byte</code>s rather than <code>long</code>s, you must exercise one of the following options:</p>

<ul>
	<li>Convert your <code>byte</code> arrays to <code>long</code> arrays, encrypt the <code>long</code> arrays, and then convert the resulting <code>long</code> arrays back into <code>byte</code> arrays.  (Class <code>lava.array.BigEndianArray</code> may be helpful.)  When encrypting, if the number of input bytes is not a multiple of 8, then you must pad the input stream in some sensible manner, and reverse the pad after decryption.  <strong>Remember, DES-ECB is a 64-bit block cipher, not a <code>byte</code> cipher.</strong></li>
	<li>Don't use this class at all.  Instead, use <code>DesEngine</code> directly and let it handle the byte-to-long-to-byte conversions as described in <code>IBlockCipher64</code>.  Your input must still be in multiples of 8 <code>byte</code>s, however, so padding may still be necessary.</li>
	<li>Don't use DES in ECB mode.  Instead, use one of the DES modes designed for <code>byte</code> stream encryption/decryption, such as DES-OFB-8 (see <code>DesOfbCoder</code>) or DES-CFB-8 (see <code>DesCfbCoder</code>).</li>
</ul>

<p><code>DesEcbCoder</code> is the base class for <code>DesEcbEncoder</code> and <code>DesEcbDecoder</code> and contains implementations common to both classes.  However, this class is abstract and cannot be instantiated because you are expected to instantiate one of the subclasses instead.  <code>DesEcbCoder</code> is public only to make the JavaDoc documentation visible.</p>

@since 2001.02.02
@author Sharky
******************************************************************************/

public abstract class DesEcbCoder
	implements ITextTransform
{



	/**********************************************************************
	Encryption engine.

	<p><b>Details:</b>  <var>engine</var> is the block cipher specified at construction time.  This can be a normal DES function, a triple-DES function, or any other type of encryption function that conforms to the <code>IBlockCipher64</code> interface.</p>
	**********************************************************************/

	final IBlockCipher64 engine;



	DesEcbCoder (IBlockCipher64 engine)
	{
		if (engine == null)
			throw new NullPointerException ("engine");
		this . engine = engine;
		beginTransform ();
	}



	/**********************************************************************
	No operation.

	<p><b>Details:</b>  This method is defined only to satisfy an <code>ITextTransform</code> interface requirement and does absolutely nothing in this implementation.</p>
	**********************************************************************/

	public final void beginTransform ()
	{
	}



	/**********************************************************************
	No operation.

	<p><b>Details:</b>  This method is defined only to satisfy an <code>ITextTransform</code> interface requirement and does absolutely nothing in this implementation.</p>

	@param dest ignored
	**********************************************************************/

	public final int completeTransform (Object dest)
	{
		return 0;
	}



	/**********************************************************************
	Returns 64.

	<p><b>Details:</b> <code>getTransformInputWidth</code> returns 64, indicating that only 64-bit input blocks are supported.</p>

	@return 64
	**********************************************************************/

	public final int getTransformInputWidth ()
	{
		return 64;
	}



	/**********************************************************************
	Returns 64.

	<p><b>Details:</b> <code>getTransformOutputWidth</code> returns 64, indicating that only 64-bit output blocks are generated.</p>

	@return 64
	**********************************************************************/

	public final int getTransformOutputWidth ()
	{
		return 64;
	}



}



