///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



import lava.konst.LONG;
import lava.security.IBlockCipher64;
import lava.security.ITextTransform;



/******************************************************************************
Base class for Cipher/Output FeedBack (C/OFB) coders.

<p><b>Details:</b>  <code>DesFbCoder</code> contains definitions that are common to the DES feedback coders, namely, the cipher feedback (CFB) and output feedback (OFB) coders.  The DES feedback coders are described in FIPS PUB 81.</p>

<p>DES normally processes data in 64-bit chunks.  However, many security applications work best with different chunk sizes.  For example, a program that encrypts files might be easier to design if it encrypted data one byte at a time, since not all files have lengths that are multiples of 8 bytes.</p>

<p>The CFB and OFB modes of DES allow the applications to use DES with symbols of any width between 1 and 64 bits.  These alternative modes are not any less secure than standard DES.  For a description of how these modes work, refer to the subclass documentation.</p>

<p>The manner in which the CFB and OFB modes utilize DES can easily be applied to other 64-bit block ciphers, such as 3DES.  Therefore, the constructors for the subclasses allow the application programmer to specify not just a <code>DesEngine</code>, but <em>any</em> <code>IBlockCipher64</code> implementation.</p>

<p><small>This base class is not usable to the application programmer in any way and is public only to make the JavaDoc comments visible.</small></p>

@author Sharky
@since 1999.08.16
******************************************************************************/

public abstract class DesFbCoder
	implements ITextTransform
{



	/**********************************************************************
	Encryption engine.

	<p><b>Details:</b>  <var>engine</var> is the block cipher specified at construction time.  This can be a normal DES function, a triple-DES function, or any other type of encryption function that conforms to the <code>IBlockCipher64</code> interface.</p>
	**********************************************************************/

	final IBlockCipher64 engine;



	/**********************************************************************
	Initialization vector.
	**********************************************************************/

	final long iv;



	/**********************************************************************
	Alternative block size.

	<p><b>Details:</b>  The is the user-<wbr>selected value of k, as defined in the FIPS publication.</p>
	**********************************************************************/

	final int k;



	/**********************************************************************
	64 minus block size.

	<p><b>Details:</b>  This value is 64 - k, precomputed and cached to enhance performance.</p>
	**********************************************************************/

	final int nk;



	/**********************************************************************
	k-bit mask.

	<p><b>Details:</b>  This value is a bit mask of k consecutive 1-set bits, right-<wbr>aligned, precomputed and stored to enhance performance.</p>
	**********************************************************************/

	final long kmask;



	/**********************************************************************
	Initializes cipher with block cipher, k, and initialization vector.

	<p><b>Details:</b>  This constructor initializes a new instance with the given block cipher, value for k (the block size), and initialization vector.</p>

	@param engine the block cipher
	@param k the block size
	@param iv the initialization vector
	**********************************************************************/

	DesFbCoder (IBlockCipher64 engine, int k, long iv)
	{
		if (engine == null)
			throw new NullPointerException ("engine");
		if (k < 1 || k > 64)
			throw new IllegalArgumentException ("k=" + k);
		this . engine = engine;
		this . k = k;
		this . iv = iv;
		nk = 64 - k;
		kmask = LONG.MAX_ULONG >>> nk;
		beginTransform ();
	}



	/**********************************************************************
	DES input block.

	<p><b>Details:</b>  This is the input to the DES function that will be used to encrypt/decrypt the next k-bit block.  Initially, this value is the user-<wbr>supplied initialization vector (IV).</p>
	**********************************************************************/

	final long[] desin = new long [1];



	/**********************************************************************
	DES output buffer.

	<p><b>Details:</b>  This is the output buffer for the DES function.  It is preallocated and retained to improve performance.</p>
	**********************************************************************/

	final long[] desout = new long [1];



	public final void beginTransform ()
	{
		desin [0] = iv;
	}



	/**********************************************************************
	ITextTransform interface method.

	<p><b>Details:</b>  See <code>ITextTransform</code> for a description of this interface method's contract.</p>

	<p>This implementation produces no output symbols because DES feedback ciphers never "lag behind."  (In other words, all output data is immediately returned from <code>continueTransform</code>.)</p>

	@param dest ignored
	@return 0
	**********************************************************************/

	public final int completeTransform (Object dest)
	{
		return 0;
	}



	/**********************************************************************
	ITextTransform interface method.

	<p><b>Details:</b>  See <code>ITextTransform</code> for a description of this interface method's contract.</p>

	<p>This implementation returns the value of k provided to the constructor.</p>

	@return k
	**********************************************************************/

	public final int getTransformInputWidth ()
	{
		return k;
	}



	/**********************************************************************
	ITextTransform interface method.

	<p><b>Details:</b>  See <code>ITextTransform</code> for a description of this interface method's contract.</p>

	<p>This implementation returns the value of k provided to the constructor.</p>

	@return k
	**********************************************************************/

	public final int getTransformOutputWidth ()
	{
		return k;
	}



}



