///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



import lava.konst.LONG;
import lava.util.BitTwiddler;



/******************************************************************************
DES key.

<p><b>Details:</b>  A DesKey defines a 64-bit DES key, 56-bits of which are used to select a DES encryption function.</p>

<p><b>bit ordering</b></p>

<p><strong>Refer to the package summary for important information regarding bit ordering.</strong></p>

@see DesEngine
@see TripleDesKey
@since 1999.08.04
@author Sharky
******************************************************************************/

public final class DesKey
{



	/**********************************************************************
	Left half of 64-bit key.
	**********************************************************************/

	final int kl;



	/**********************************************************************
	Right half of 64-bit key.
	**********************************************************************/

	final int kr;



	/**********************************************************************
	Subkeys used for encoding.
	**********************************************************************/

	final int[] ekl = new int [16];
	final int[] ekr = new int [16];



	/**********************************************************************
	Subkeys used for decoding.
	**********************************************************************/

	final int[] dkl = new int [16];
	final int[] dkr = new int [16];



	/**********************************************************************
	Sets key halves.

	<p><b>Details:</b>  This constructor initializes the key using the 32-bit left and right halves provided.  The MSB of <var>lh</var> corresponds to bit 1 of the key described in FIPS PUB 46, while the LSB of <var>rh</var> corresponds to bit 64.</p>

	@param lh the left half
	@param rh the right half
	**********************************************************************/

	public DesKey (int lh, int rh)
	{
		this . kl = lh;
		this . kr = rh;
		updateSubkeys ();
	}



	/**********************************************************************
	Sets key.

	<p><b>Details:</b>  This constructor initializes the key using the given 64-bit value.  The MSB corresponds to bit 1 of the key described in FIPS PUB 46.</p>

	@param key the key
	**********************************************************************/

	public DesKey (long key)
	{
		this
		(
			(int) (key >> 32),
			(int)  key
		);
	}



	/**********************************************************************
	Specifies 32-bit left and right halves.

	<p><b>Details:</b>  This constructor initializes the key using the 32-bit left and right halves provided.  The left half is index by <var>offset</var> and the right half is index by <var>offset</var> + 1.</p>

	@param halves the left and right halves
	@param offset index of left half
	**********************************************************************/

	public DesKey (int[] halves, int offset)
	{
		this
		(
			halves [offset],
			halves [offset + 1]
		);
	}



	/**********************************************************************
	Specifies key as a byte sequence.

	<p><b>Details:</b>  This constructor initializes the key using the given 8 bytes.  The bytes are ordered in big endian order, and the first byte is indexed by <var>offset</var>, and the MSB of the first byte corresponds to bit 1 of the key described in FIPS PUB 46.</p>

	@param bytes the byte array
	@param offset index of first byte
	**********************************************************************/

	public DesKey (byte[] bytes, int offset)
	{
		this
		(
			BitTwiddler.toIntBe (bytes, offset    ),
			BitTwiddler.toIntBe (bytes, offset + 4)
		);
	}



	/**********************************************************************
	Initializes subkeys.

	<p><b>Details:</b>  This method initializes the encryption and decryption subkeys according to the values stored in <var>lh</var> and <var>rh</var>.</p>
	**********************************************************************/

	private void updateSubkeys ()
	{
		int resl =
			(kl & 0x80000000) >>> 11
		|	(kl & 0x40000000) >>> 18
		|	(kl & 0x20000000) >>> 25
		|	(kl & 0x00000080)  << 16
		|	(kl & 0x00400000) >>>  9
		|	(kl & 0x00200000) >>> 16
		|	(kl & 0x00002000) >>>  7
		|	(kl & 0x00800000) >>>  2
		|	 kl & 0x00004000
		|	(kl & 0x00000020)  <<  2
		|	(kl & 0x00008000)  <<  7
		|	(kl & 0x00000040)  <<  9
		|	(kr & 0x10000000) >>> 28
		|	(kr & 0x20000000) >>> 21
		|	(kr & 0x00100000) >>> 19
		|	(kr & 0x40000000) >>> 14
		|	(kr & 0x00200000) >>> 12
		|	(kr & 0x00001000) >>> 10
		|	(kr & 0x80000000) >>>  7
		|	(kr & 0x00400000) >>>  5
		|	(kr & 0x00002000) >>>  3
		|	(kr & 0x00000010) >>>  1
		|	(kr & 0x00800000)  <<  2
		|	(kr & 0x00004000)  <<  4
		|	(kr & 0x00000020)  <<  6
		|	(kr & 0x00008000)  << 11
		|	(kr & 0x00000040)  << 13
		|	(kr & 0x00000080)  << 20;
		int resr =
			(kl & 0x10000000) >>> 28
		|	(kl & 0x08000000) >>> 23
		|	(kl & 0x00100000) >>> 19
		|	(kl & 0x04080000) >>> 14
		|	(kl & 0x00001000) >>> 10
		|	(kl & 0x02040800) >>>  5
		|	(kl & 0x00000010) >>>  1
		|	(kl & 0x00020408)  <<  4
		|	(kl & 0x00000204)  << 13
		|	(kl & 0x00000002)  << 22
		|	(kr & 0x08000000) >>> 19
		|	(kr & 0x04080000) >>> 10
		|	(kr & 0x02040800) >>>  1
		|	(kr & 0x00020408)  <<  8
		|	(kr & 0x00000204)  << 17
		|	(kr & 0x00000002)  << 26;
		resl = resl << 1 | resl >>> (28 - 1);
		resr = resr << 1 | resr >>> (28 - 1);
		final_key_compression_permutation (0, resl, resr);
		resl = resl << 1 | resl >>> (28 - 1);
		resr = resr << 1 | resr >>> (28 - 1);
		final_key_compression_permutation (1, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (2, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (3, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (4, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (5, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (6, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (7, resl, resr);
		resl = resl << 1 | resl >>> (28 - 1);
		resr = resr << 1 | resr >>> (28 - 1);
		final_key_compression_permutation (8, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (9, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (10, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (11, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (12, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (13, resl, resr);
		resl = resl << 2 | resl >>> (28 - 2);
		resr = resr << 2 | resr >>> (28 - 2);
		final_key_compression_permutation (14, resl, resr);
		resl = resl << 1 | resl >>> (28 - 1);
		resr = resr << 1 | resr >>> (28 - 1);
		final_key_compression_permutation (15, resl, resr);
	}



	/**********************************************************************
	Performs final key compression permutation.

	<p><b>Details:</b>  This method performs the final key compression operation on the given rotated left and right halves to generate a subkey.  The result is not returned but is stored in the subkey arrays.</p>

	@param index the subkey index
	@param resl the left half of the rotated key
	@param resr the right half of the rotated key
	**********************************************************************/

	private void final_key_compression_permutation (int index, int resl, int resr)
	{
		ekl [index] =
		dkl [15 - index] =
			(resl & 0x04000000) >>> 26
		|	(resl & 0x00200000) >>> 17
		|	(resl & 0x01000000) >>> 16
		|	(resl & 0x00108000) >>> 14
		|	(resl & 0x0a400000) >>>  8
		|	(resl & 0x00011000) >>>  7
		|	(resl & 0x00040100) >>>  6
		|	(resl & 0x00800000) >>>  5
		|	(resl & 0x00000200)  <<  1
		|	(resl & 0x00002002)  <<  2
		|	(resl & 0x00020000)  <<  4
		|	(resl & 0x00000004)  <<  5
		|	(resl & 0x000000a0)  <<  6
		|	(resl & 0x00004000)  <<  9
		|	(resl & 0x00000800)  << 11
		|	(resl & 0x00000011)  << 16;
		ekr [index] =
		dkr [15 - index] =
			(resr & 0x08000000) >>> 26
		|	(resr & 0x01000000) >>> 24
		|	(resr & 0x00100000) >>> 18
		|	(resr & 0x00400000) >>> 15
		|	(resr & 0x00804000) >>> 10
		|	(resr & 0x04000000) >>>  9
		|	(resr & 0x00020000) >>>  8
		|	(resr & 0x00000400) >>>  5
		|	(resr & 0x02000000) >>>  4
		|	(resr & 0x00000040) >>>  3
		|	(resr & 0x00001000) >>>  1
		|	 resr & 0x00010000
		|	(resr & 0x00080000)  <<  1
		|	(resr & 0x00000888)  <<  3
		|	(resr & 0x00000100)  <<  4
		|	(resr & 0x00008001)  <<  8
		|	(resr & 0x00000220)  << 10
		|	(resr & 0x00000002)  << 17
		|	(resr & 0x00000010)  << 18;
	}



	/**********************************************************************
	Tests for odd parity.

	<p><b>Details:</b>  This method determines whether this DES key has odd parity.  A DES key is said to have odd parity when each octet (byte) of the key has odd parity (i.e., total number of set bits is odd).</p>

	<p>A correctly specified DES key should have odd parity.</p>

	@return true iff parity is odd
	**********************************************************************/

	public boolean hasOddParity ()
	{
		byte[] b =
			BitTwiddler.toBytesBe
			(
				kr,
				BitTwiddler.toBytesBe (kl, new byte [8], 0),
				4
			);
		for (int i = 0; i < 8; ++ i)
			if (BitTwiddler.countSetBits (b [i]) % 2 == 0)
				return false;
		return true;
	}



	/**********************************************************************
	Converts this key to long value.

	<p><b>Details:</b>  This method returns this key as a <code>long</code> value.</p>

	@return the long value
	@since 1999.08.17
	**********************************************************************/

	public long longValue ()
	{
		return (long) kl << 32 | kr & LONG.MAX_UINT;
	}



	/**********************************************************************
	Converts this key to a String.

	<p><b>Details:</b>  This method returns this key as a 16 character hexadecimal <code>String</code>.</p>

	@return the String
	@since 1999.08.17
	**********************************************************************/

	public String toString ()
	{
		String s = "0000000000000000" + Long.toHexString (longValue ());
		return s . substring (s . length () - 16);
	}



}



