///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



import lava.security.IBlockCipher64;



/******************************************************************************
Optimized 3DES implementation.

<p><b>Details:</b>  TripleDesEngine is an efficient implementation of the 3DES chain cipher.  A 3DES cipher is simply three, independently-<wbr>keyed DES ciphers, operating in ECB mode, that are chained together.  In 3DES, the data is encrypted using the first key, decrypted using the second key, and encrypted again using the third key.  The reverse process (decrypt-<wbr>encrypt-<wbr>decrypt) recovers the data.</p>

<p>Except for key initialization, the interface to this class is identical to DesEngine.  Make sure to read the documentation for class {@link DesEngine DesEngine} before using this class.</p>

@see DesEngine
@since 1999.08.03
@version 2001.03.09
******************************************************************************/

public final class TripleDesEngine
	implements IBlockCipher64
{



	/**********************************************************************
	Encryption key.
	**********************************************************************/

	private final TripleDesKey key;



	/**********************************************************************
	First DES cipher.
	**********************************************************************/

	private final DesEngine des1;



	/**********************************************************************
	Second DES cipher.
	**********************************************************************/

	private final DesEngine des2;



	/**********************************************************************
	Third DES cipher.
	**********************************************************************/

	private final DesEngine des3;



	/**********************************************************************
	Sets triple-key.

	<p><b>Details:</b>  This constructor initializes a new <code>TripleDesEngine</code> instance for standard 3DES encryption/decryption.</p>

	@param key the 3DES key
	**********************************************************************/

	public TripleDesEngine (TripleDesKey key)
	{
		this (key, true, true);
	}



	/**********************************************************************
	Sets triple-key; enables/disables permutations.

	<p><b>Details:</b>  This constructor initializes a new <code>TripleDesEngine</code> instance and enables or disables initial and final text permutations.</p>

	@param key the 3DES key
	@param itpe initial text permutation enabled
	@param ftpe final text permutation enabled
	**********************************************************************/

	public TripleDesEngine (TripleDesKey key, boolean itpe, boolean ftpe)
	{
		this . key = key;
		des1 = new DesEngine (key . key1, itpe , false);
		des2 = new DesEngine (key . key2, false, false);
		des3 = new DesEngine (key . key3, false, ftpe );
	}



	/**********************************************************************
	Encrypts blocks.

	<p><b>Details:</b></p>  See IBlockCipher64 (read all of it!) for a description of this interface method's contract.</p>

	@param src source array
	@param dest destination array
	@param srclen number of elements in source array
	**********************************************************************/

	public final void encrypt (Object src, Object dest, int srclen)
	{
		crypt (true, src, dest, srclen);
	}



	/**********************************************************************
	Decrypts blocks.

	<p><b>Details:</b></p>  See IBlockCipher64 (read all of it!) for a description of this interface method's contract.</p>

	@param src source array
	@param dest destination array
	@param srclen number of elements in source array
	**********************************************************************/

	public final void decrypt (Object src, Object dest, int srclen)
	{
		crypt (false, src, dest, srclen);
	}



	/**********************************************************************
	Encrypts or decrypts blocks of data.

	<p><b>Details:</b>  This method encrypts or decrypts a vector of blocks depending on the value of <var>forward</var>.</p>

	@param forward whether to encrypt (true) or decrypt (false)
	@param input source array
	@param output destination array
	@param srclen number of elements in source array
	**********************************************************************/

	private void crypt (boolean forward, Object input, Object output, int srclen)
	{
		if (srclen < 0)
			throw new IllegalArgumentException ("srclen=" + srclen);
		// Factored out for speed!
		if (input instanceof byte[])
			if (output instanceof byte[])
				crypt (forward, (byte []) input, (byte []) output, srclen);
			else if (output instanceof int[])
				crypt (forward, (byte []) input, (int  []) output, srclen);
			else if (output instanceof long[])
				crypt (forward, (byte []) input, (long []) output, srclen);
			else if (output instanceof short[])
				crypt (forward, (byte []) input, (short[]) output, srclen);
			else
				throw new IllegalArgumentException ("output");
		else if (input instanceof int[])
			if (output instanceof int[])
				crypt (forward, (int  []) input, (int  []) output, srclen);
			else if (output instanceof byte[])
				crypt (forward, (int  []) input, (byte []) output, srclen);
			else if (output instanceof long[])
				crypt (forward, (int  []) input, (long []) output, srclen);
			else if (output instanceof short[])
				crypt (forward, (int  []) input, (short[]) output, srclen);
			else
				throw new IllegalArgumentException ("output");
		else if (input instanceof long[])
			if (output instanceof long[])
				crypt (forward, (long []) input, (long []) output, srclen);
			else if (output instanceof int[])
				crypt (forward, (long []) input, (int  []) output, srclen);
			else if (output instanceof byte[])
				crypt (forward, (long []) input, (byte []) output, srclen);
			else if (output instanceof short[])
				crypt (forward, (long []) input, (short[]) output, srclen);
			else
				throw new IllegalArgumentException ("output");
		else if (input instanceof short[])
			if (output instanceof short[])
				crypt (forward, (short[]) input, (short[]) output, srclen);
			else if (output instanceof byte[])
				crypt (forward, (short[]) input, (byte []) output, srclen);
			else if (output instanceof int[])
				crypt (forward, (short[]) input, (int  []) output, srclen);
			else if (output instanceof long[])
				crypt (forward, (short[]) input, (long []) output, srclen);
			else
				throw new IllegalArgumentException ("output");
		else
			throw new IllegalArgumentException ("input");
	}



	private void crypt (boolean forward, long[] in64, long[] out64, int length)
	{
		for (int i = 0; i < length; ++ i)
			out64 [i] = crypt (forward, (int) (in64 [i] >> 32), (int) in64 [i]);
	}



	private void crypt (boolean forward, long[] in64, int[] out32, int length)
	{
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 1, o += 2)
		{
			text = crypt (forward, (int) (in64 [i] >> 32), (int) in64 [i]);
			out32 [o    ] = (int) (text >> 32);
			out32 [o + 1] = (int)  text       ;
		}
	}



	private void crypt (boolean forward, long[] in64, short[] out16, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 1, o += 4)
		{
			text = crypt (forward, (int) (in64 [i] >> 32), (int) in64 [i]);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out16 [o    ] = (short) (l0 >> 16);
			out16 [o + 1] = (short)  l0       ;
			out16 [o + 2] = (short) (r0 >> 16);
			out16 [o + 3] = (short)  r0       ;
		}
	}



	private void crypt (boolean forward, long[] in64, byte[] out08, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 1, o += 8)
		{
			text = crypt (forward, (int) (in64 [i] >> 32), (int) in64 [i]);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out08 [o    ] = (byte) (l0 >> 24);
			out08 [o + 1] = (byte) (l0 >> 16);
			out08 [o + 2] = (byte) (l0 >>  8);
			out08 [o + 3] = (byte)  l0       ;
			out08 [o + 4] = (byte) (r0 >> 24);
			out08 [o + 5] = (byte) (r0 >> 16);
			out08 [o + 6] = (byte) (r0 >>  8);
			out08 [o + 7] = (byte)  r0       ;
		}
	}



	private void crypt (boolean forward, int[] in32, long[] out64, int length)
	{
		int o = 0;
		for (int i = 0; i < length; i += 2, o += 1)
			out64 [o] = crypt (forward, in32 [i], in32 [i + 1]);
	}



	private void crypt (boolean forward, int[] in32, int[] out32, int length)
	{
		long text;
		for (int i = 0; i < length; i += 2)
		{
			text = crypt (forward, in32 [i], in32 [i + 1]);
			out32 [i    ] = (int) (text >> 32);
			out32 [i + 1] = (int)  text       ;
		}
	}



	private void crypt (boolean forward, int[] in32, short[] out16, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 2, o += 4)
		{
			text = crypt (forward, in32 [i], in32 [i + 1]);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out16 [o    ] = (short) (l0 >> 16);
			out16 [o + 1] = (short)  l0       ;
			out16 [o + 2] = (short) (r0 >> 16);
			out16 [o + 3] = (short)  r0       ;
		}
	}



	private void crypt (boolean forward, int[] in32, byte[] out08, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 2, o += 8)
		{
			text = crypt (forward, in32 [i], in32 [i + 1]);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out08 [o    ] = (byte) (l0 >> 24);
			out08 [o + 1] = (byte) (l0 >> 16);
			out08 [o + 2] = (byte) (l0 >>  8);
			out08 [o + 3] = (byte)  l0       ;
			out08 [o + 4] = (byte) (r0 >> 24);
			out08 [o + 5] = (byte) (r0 >> 16);
			out08 [o + 6] = (byte) (r0 >>  8);
			out08 [o + 7] = (byte)  r0       ;
		}
	}




	private void crypt (boolean forward, short[] in16, long[] out64, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 4, o += 1)
			out64 [o] = crypt
			(	forward,
				in16 [i    ] << 16 | (in16 [i + 1] & 0xffff),
				in16 [i + 2] << 16 | (in16 [i + 3] & 0xffff)
			);
	}



	private void crypt (boolean forward, short[] in16, int[] out32, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 4, o += 2)
		{
			text = crypt
			(	forward,
				in16 [i    ] << 16 | (in16 [i + 1] & 0xffff),
				in16 [i + 2] << 16 | (in16 [i + 3] & 0xffff)
			);
			out32 [o    ] = (int) (text >> 32);
			out32 [o + 1] = (int)  text       ;
		}
	}



	private void crypt (boolean forward, short[] in16, short[] out16, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 4, o += 4)
		{
			text = crypt
			(	forward,
				in16 [i    ] << 16 | (in16 [i + 1] & 0xffff),
				in16 [i + 2] << 16 | (in16 [i + 3] & 0xffff)
			);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out16 [o    ] = (short) (l0 >> 16);
			out16 [o + 1] = (short)  l0       ;
			out16 [o + 2] = (short) (r0 >> 16);
			out16 [o + 3] = (short)  r0       ;
		}
	}



	private void crypt (boolean forward, short[] in16, byte[] out08, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 4, o += 8)
		{
			text = crypt
			(	forward,
				in16 [i    ] << 16 | (in16 [i + 1] & 0xffff),
				in16 [i + 2] << 16 | (in16 [i + 3] & 0xffff)
			);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out08 [o    ] = (byte) (l0 >> 24);
			out08 [o + 1] = (byte) (l0 >> 16);
			out08 [o + 2] = (byte) (l0 >>  8);
			out08 [o + 3] = (byte)  l0       ;
			out08 [o + 4] = (byte) (r0 >> 24);
			out08 [o + 5] = (byte) (r0 >> 16);
			out08 [o + 6] = (byte) (r0 >>  8);
			out08 [o + 7] = (byte)  r0       ;
		}
	}



	private void crypt (boolean forward, byte[] in08, long[] out64, int length)
	{
		int o = 0;
		for (int i = 0; i < length; i += 8, o += 1)
			out64 [o] = crypt
			(	forward,
				 in08 [i    ]         << 24
			|	(in08 [i + 1] & 0xff) << 16
			|	(in08 [i + 2] & 0xff) <<  8
			|	(in08 [i + 3] & 0xff)      ,
				 in08 [i + 4]         << 24
			|	(in08 [i + 5] & 0xff) << 16
			|	(in08 [i + 6] & 0xff) <<  8
			|	(in08 [i + 7] & 0xff)
			);
	}



	private void crypt (boolean forward, byte[] in08, int[] out32, int length)
	{
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 8, o += 2)
		{
			text = crypt
			(	forward,
				 in08 [i    ]         << 24
			|	(in08 [i + 1] & 0xff) << 16
			|	(in08 [i + 2] & 0xff) <<  8
			|	(in08 [i + 3] & 0xff)      ,
				 in08 [i + 4]         << 24
			|	(in08 [i + 5] & 0xff) << 16
			|	(in08 [i + 6] & 0xff) <<  8
			|	(in08 [i + 7] & 0xff)
			);
			out32 [o    ] = (int) (text >> 32);
			out32 [o + 1] = (int)  text       ;
		}
	}



	private void crypt (boolean forward, byte[] in08, short[] out16, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 8, o += 4)
		{
			text = crypt
			(	forward,
				 in08 [i    ]         << 24
			|	(in08 [i + 1] & 0xff) << 16
			|	(in08 [i + 2] & 0xff) <<  8
			|	(in08 [i + 3] & 0xff)      ,
				 in08 [i + 4]         << 24
			|	(in08 [i + 5] & 0xff) << 16
			|	(in08 [i + 6] & 0xff) <<  8
			|	(in08 [i + 7] & 0xff)
			);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out16 [o    ] = (short) (l0 >> 16);
			out16 [o + 1] = (short)  l0       ;
			out16 [o + 2] = (short) (r0 >> 16);
			out16 [o + 3] = (short)  r0       ;
		}
	}



	private void crypt (boolean forward, byte[] in08, byte[] out08, int length)
	{
		int l0, r0;
		long text;
		int o = 0;
		for (int i = 0; i < length; i += 8, o += 8)
		{
			text = crypt
			(	forward,
				 in08 [i    ]         << 24
			|	(in08 [i + 1] & 0xff) << 16
			|	(in08 [i + 2] & 0xff) <<  8
			|	(in08 [i + 3] & 0xff)      ,
				 in08 [i + 4]         << 24
			|	(in08 [i + 5] & 0xff) << 16
			|	(in08 [i + 6] & 0xff) <<  8
			|	(in08 [i + 7] & 0xff)
			);
			l0 = (int) (text >> 32);
			r0 = (int)  text       ;
			out08 [o    ] = (byte) (l0 >> 24);
			out08 [o + 1] = (byte) (l0 >> 16);
			out08 [o + 2] = (byte) (l0 >>  8);
			out08 [o + 3] = (byte)  l0       ;
			out08 [o + 4] = (byte) (r0 >> 24);
			out08 [o + 5] = (byte) (r0 >> 16);
			out08 [o + 6] = (byte) (r0 >>  8);
			out08 [o + 7] = (byte)  r0       ;
		}
	}



	long crypt (boolean forward, int l0, int r0)
	{
		if (forward)
		{
			long text = des1 . crypt (true, l0, r0);
			text = des2 . crypt (false, (int) (text >>> 32), (int) text);
			return des3 . crypt (true, (int) (text >>> 32), (int) text);
		}
		else
		{
			long text = des3 . crypt (false, l0, r0);
			text = des2 . crypt (true, (int) (text >>> 32), (int) text);
			return des1 . crypt (false, (int) (text >>> 32), (int) text);
		}
	}



}



