///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.util;



/******************************************************************************
Bit-twiddling operations.

<p><code>BitTwiddler</code> is a collection of static methods that perform bit-twiddling operations.</p>

@version 1999.08.08
******************************************************************************/

public class BitTwiddler
{



	///////////////////////////////////////
	//                                   //
	//  byte[]-to-primitive conversions  //
	//                                   //
	///////////////////////////////////////



	/**********************************************************************
	Converts big endian bytes into long.

	<p><b>Details:</b>  This method converts big endian ordered <code>byte</code>s into a <code>long</code>.  <var>offset</var> is the array index of the first byte.</p>

	@param b the byte array
	@param the array offset (<b>default value:</b> 0)
	@return the long

	@since 1999.08.01
	**********************************************************************/

	public static final long toLongBe (byte[] b, int offset)
	{
		return
			(b [offset + 0] & 0xffL) << (7 * 8)
		|	(b [offset + 1] & 0xffL) << (6 * 8)
		|	(b [offset + 2] & 0xffL) << (5 * 8)
		|	(b [offset + 3] & 0xffL) << (4 * 8)
		|	(b [offset + 4] & 0xffL) << (3 * 8)
		|	(b [offset + 5] & 0xffL) << (2 * 8)
		|	(b [offset + 6] & 0xffL) << (1 * 8)
		|	(b [offset + 7] & 0xffL);
	}



	/**********************************************************************
	Converts big endian bytes into int.

	<p><b>Details:</b>  This method converts big endian ordered <code>byte</code>s into an <code>int</code>.  <var>offset</var> is the array index of the first byte.</p>

	@param b the byte array
	@param the array offset (<b>default value:</b> 0)
	@return the int

	@since 1999.08.01
	**********************************************************************/

	public static final int toIntBe (byte[] b, int offset)
	{
		return
			(b [offset + 0] & 0xff) << (3 * 8)
		|	(b [offset + 1] & 0xff) << (2 * 8)
		|	(b [offset + 2] & 0xff) << (1 * 8)
		|	 b [offset + 3] & 0xff            ;
	}



	/**********************************************************************
	Breaks long into big endian bytes.

	<p><b>Details:</b>  This method breaks a <code>long</code> into 8 <code>byte</code>s and stores them in the given array, at the given index, using big endian byte ordering.</p>

	@param l the long
	@param dest the destination array
	@param offset the array index
	@return dest
	**********************************************************************/

	public static final byte[] toBytesBe (long l, byte[] dest, int offset)
	{
		dest [offset + 0] = (byte) (l >> 7 * 8);
		dest [offset + 1] = (byte) (l >> 6 * 8);
		dest [offset + 2] = (byte) (l >> 5 * 8);
		dest [offset + 3] = (byte) (l >> 4 * 8);
		dest [offset + 4] = (byte) (l >> 3 * 8);
		dest [offset + 5] = (byte) (l >> 2 * 8);
		dest [offset + 6] = (byte) (l >>     8);
		dest [offset + 7] = (byte)  l          ;
		return dest;
	}



	/**********************************************************************
	Counts set bits.

	<p><b>Details:</b>  This method returns the number of bits in <var>i</var> that are 1.</p>

	@param i an int
	@return the count
	@since 1999.08.08
	**********************************************************************/

	public static final int countSetBits (int i)
	{
		int count = 0;
		while (i != 0)
		{
			if ((i & 1) != 0)
				++ count;
			i >>>= 1;
		}
		return count;
	}



	/**********************************************************************
	Counts set bits.

	<p><b>Details:</b>  This method returns the number of bits in <var>b</var> that are 1.</p>

	@param b a byte
	@return the count
	@since 1999.08.08
	**********************************************************************/

	public static final int countSetBits (byte b)
	{
		return countSetBits (b & 0xff);
	}



}



